package meerkat.crypto.concrete;

import meerkat.protobuf.BulletinBoardAPI;
import meerkat.protobuf.ConcreteCrypto;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Voting;
import org.junit.Before;
import org.junit.Test;
import qilin.primitives.concrete.ECElGamal;
import qilin.primitives.concrete.ECGroup;

import java.math.BigInteger;
import java.util.Random;

/**
 * Test class for {@link ECElGamalEncryption}
 */
public class ECElGamalEncryptionTest {
    Random rand = new Random(0); // Insecure deterministic random for testing.

    ECElGamal.SK key;
    ECGroup group;
    ECElGamalEncryption enc;
    ConcreteCrypto.ElGamalPublicKey serializedPk;


    @Before
    public void setup() throws Exception {
        group = new ECGroup("secp256k1");
        BigInteger sk = ECElGamal.generateSecretKey(group, rand);
        key = new ECElGamal.SK(group, sk);
        serializedPk = ECElGamalUtils.serializePk(group, key);


        enc = new ECElGamalEncryption();

        enc.init(serializedPk);
    }

    @Test
    public void testEncrypt() throws Exception {
        Voting.PlaintextBallot msg = Voting.PlaintextBallot.newBuilder()
                .setSerialNumber(1)
                .addAnswers( Voting.BallotAnswer.newBuilder()
                    .addAnswer(2)
                    .addAnswer(3)
                    .addAnswer(0)
                )
                .addAnswers( Voting.BallotAnswer.newBuilder()
                        .addAnswer(5)
                        .addAnswer(6)
                        .addAnswer(7)
                )
                .build();

        Crypto.RerandomizableEncryptedMessage cipherText = enc.encrypt(msg, enc.generateRandomness(rand));

        Voting.PlaintextBallot decrypted = ECElGamalUtils.decrypt(Voting.PlaintextBallot.class, key, group, cipherText);

    }
}
